#!/bin/bash -e

# https://mikefarah.gitbook.io/yq/commands/write-update
YQ_MAIN_VERSION="$(./yq --version|sed -e 's/^.\+version \([0-9]\)\..*/\1/')"

function yq_read() {
    eval "yq_${YQ_MAIN_VERSION}_read $*"
}
function yq_write() {
    eval "yq_${YQ_MAIN_VERSION}_write $*"
}
function yq_add_map() {
  eval "yq_${YQ_MAIN_VERSION}_add_map $*"
}
function yq_4_read() {
    YML_FILE=$1
    ITEM=$2
    #yq eval '.services.rm-proxy-svc.environment' docker-compose.yml
    ./yq e ".$ITEM" "$YML_FILE"
}
function yq_4_write() {
    YML_FILE=$1
    ITEM=$2
    VALUE=$3
    #yq eval '.services.rm-proxy-svc.environment' docker-compose.yml
    #yq e '.a.b += ["cow"]' 
    ./yq -i e ".$ITEM += [\"$VALUE\"]" "$YML_FILE"
}
function yq_4_add_map() {
    YML_FILE=$1
    ITEM=$2
    VALUE=$3
    #yq -i e ".services.deployment-init.user = \"root\""
    ./yq -i e ".$ITEM = \"$VALUE\"" "$YML_FILE"
}
function yq_3_read() {
    YML_FILE=$1
    ITEM=$2
    #yq r "$PLATFORM_YML" services.deployment-init.environment
    ./yq r "$YML_FILE" "$ITEM"
}
function yq_3_write() {
    YML_FILE=$1
    ITEM=$2
    VALUE=$3
    #yq w -i "$PLATFORM_YML" "services.deployment-init.environment[+]" "HTTPS_CRT_PATH=\${HTTPS_CRT_PATH}"
    ./yq w -i "$YML_FILE" "$ITEM[+]" "$VALUE"
}
function yq_3_add_map() {
    YML_FILE=$1
    ITEM=$2
    VALUE=$3
    #yq w -i docker-compose.yml "services.deployment-init.user" "root"
    ./yq w -i "$YML_FILE" "$ITEM" "$VALUE"
}

PLATFORM_YML="docker-compose.yml"
ENV_FILE=".env"

PLATFORM_ORIG="docker-compose_orig.yml"
ENV_ORIG=".env_orig"

if [ -f "$PLATFORM_ORIG" ];then
    echo "Backup file $PLATFORM_ORIG already created"
else
    cp "$PLATFORM_YML" "$PLATFORM_ORIG"
fi
if [ -f "$ENV_ORIG" ];then
    echo "Backup file $ENV_ORIG already created"
else
    cp "$ENV_FILE" "$ENV_ORIG"
fi

if [ "$1" == "extraKC" ]
then
    if ! (yq_read "$PLATFORM_YML" services.keycloak.volumes|grep -v "^#"|grep -q '\.\/ssl\/java_cacerts\/cacerts');then
        yq_write "$PLATFORM_YML" "services.keycloak.volumes" "./ssl/java_cacerts/cacerts:/tmp/cacerts"
    fi
    if ! (yq_read "$PLATFORM_YML" services.keycloak|grep -v "^#"|grep -q '--spi-truststore-file-file=/tmp/cacerts');then
        yq_write "$PLATFORM_YML" "services.keycloak.entrypoint" '/opt/keycloak/bin/kc.sh'
        yq_write "$PLATFORM_YML" "services.keycloak.entrypoint" 'start' 
        yq_write "$PLATFORM_YML" "services.keycloak.entrypoint" '--spi-truststore-file-file=/tmp/cacerts'
        yq_write "$PLATFORM_YML" "services.keycloak.entrypoint" '--spi-truststore-file-password=changeit'
        yq_write "$PLATFORM_YML" "services.keycloak.entrypoint" '--spi-truststore-file-hostname-verification-policy=ANY'
    fi
else
    if ! (yq_read "$PLATFORM_YML" "services.deployment-init.environment"|grep -v "^#"|grep -q HTTPS_CRT_PATH);then
        yq_write "$PLATFORM_YML" "services.deployment-init.environment" 'HTTPS_CRT_PATH=\${HTTPS_CRT_PATH}'
    fi
    if ! (yq_read "$PLATFORM_YML" "services.deployment-init"|grep -v "^#"|grep -q root);then
        yq_add_map "$PLATFORM_YML" "services.deployment-init.user" "root"
    fi
    if ! (yq_read "$PLATFORM_YML" services.deployment-init.volumes|grep -v "^#"|grep -q '\/tmp\/ssl');then
        yq_write "$PLATFORM_YML" "services.deployment-init.volumes" "./ssl:/tmp/ssl"
    fi
    if ! (yq_read "$PLATFORM_YML" services.aihub-job-agent.environment|grep -v "^#"|grep -q 'JAVA_KEYSTORE_PATH');then
        yq_write "$PLATFORM_YML" "services.aihub-job-agent.environment" "JAVA_KEYSTORE_PATH=/usr/lib/jvm/java-11-openjdk-amd64/lib/security/cacerts"
    fi
    if ! (yq_read "$PLATFORM_YML" services.aihub-job-agent.volumes|grep -v "^#"|grep -q '\/java_cacerts');then
        yq_write "$PLATFORM_YML" "services.aihub-job-agent.volumes" ./ssl/java_cacerts/cacerts:/usr/lib/jvm/java-11-openjdk-amd64/lib/security/cacerts
    fi
    if ! (yq_read "$PLATFORM_YML" services.scoring-agent.environment|grep -v "^#"|grep -q 'JAVA_KEYSTORE_PATH');then
        yq_write "$PLATFORM_YML" "services.scoring-agent.environment" "JAVA_KEYSTORE_PATH=/usr/lib/jvm/java-11-openjdk-amd64/lib/security/cacerts"
    fi
    if ! (yq_read "$PLATFORM_YML" services.scoring-agent.volumes|grep -v "^#"|grep -q '\/java_cacerts');then
        yq_write "$PLATFORM_YML" "services.scoring-agent.volumes" ./ssl/java_cacerts/cacerts:/usr/lib/jvm/java-11-openjdk-amd64/lib/security/cacerts
    fi
    if ! (yq_read "$PLATFORM_YML" services.aihub-backend.environment|grep -v "^#"|grep -q 'JAVA_KEYSTORE_PATH');then
        yq_write "$PLATFORM_YML" "services.aihub-backend.environment" "JAVA_KEYSTORE_PATH=/usr/lib/jvm/java-11-openjdk-amd64/lib/security/cacerts"
    fi
    if ! (yq_read "$PLATFORM_YML" services.aihub-backend.volumes|grep -v "^#"|grep -q '\/java_cacerts');then
        yq_write "$PLATFORM_YML" "services.aihub-backend.volumes" "./ssl/java_cacerts/cacerts:/usr/lib/jvm/java-11-openjdk-amd64/lib/security/cacerts"
    fi
    # if ! (yq_read "$PLATFORM_YML" services.radoop-proxy.volumes|grep -v "^#"|grep -q '\/ca-bunde');then
    #     yq_write "$PLATFORM_YML" "services.radoop-proxy.volumes" "./ssl/rh_cacerts/ca-bundle.trust.crt:/etc/pki/ca-trust/extracted/openssl/ca-bundle.trust.crt"
    #     yq_write "$PLATFORM_YML" "services.radoop-proxy.volumes" "./ssl/rh_cacerts/ca-bundle.crt:/etc/pki/tls/certs/ca-bundle.crt"
    # fi
    if ! (yq_read "$PLATFORM_YML" services.platform-admin.volumes|grep -v "^#"|grep -q '\.\/ssl\/deb_cacerts');then
        yq_write "$PLATFORM_YML" "services.platform-admin.volumes" "./ssl/deb_cacerts/:/etc/ssl/certs/"
    fi
    if ! (yq_read "$PLATFORM_YML" services.jupyterhub.environment|grep -v "^#"|grep -q 'REQUESTS_CA_BUNDLE');then
        yq_write "$PLATFORM_YML" "services.jupyterhub.environment" "REQUESTS_CA_BUNDLE=/etc/ssl/certs/ca-certificates.crt"
    fi
    if ! (yq_read "$PLATFORM_YML" services.jupyterhub.environment|grep -v "^#"|grep -q 'JUPYTERHUB_CUSTOM_CA_CERTS');then
        yq_write "$PLATFORM_YML" "services.jupyterhub.environment" 'JUPYTERHUB_CUSTOM_CA_CERTS=\${JUPYTERHUB_CUSTOM_CA_CERTS}'
    fi
    if ! (yq_read "$PLATFORM_YML" services.jupyterhub.volumes|grep -v "^#"|grep -q '\.\/ssl\/deb_cacerts');then
        yq_write "$PLATFORM_YML" "services.jupyterhub.volumes" "./ssl/deb_cacerts/:/etc/ssl/certs/"
    fi
    if ! (yq_read "$PLATFORM_YML" services.grafana.volumes|grep -v "^#"|grep -q '\.\/ssl\/deb_cacerts');then
        yq_write "$PLATFORM_YML" "services.grafana.volumes" "./ssl/deb_cacerts/:/etc/ssl/certs/"
    fi
    if ! (yq_read "$PLATFORM_YML" services.landing-page.volumes|grep -v "^#"|grep -q '\.\/ssl\/deb_cacerts');then
        yq_write "$PLATFORM_YML" "services.landing-page.volumes" "./ssl/deb_cacerts/:/etc/ssl/certs/"
    fi
    if ! (yq_read "$PLATFORM_YML" services.token-tool.volumes|grep -v "^#"|grep -q '\.\/ssl\/deb_cacerts');then
        yq_write "$PLATFORM_YML" "services.token-tool.volumes" "./ssl/deb_cacerts/:/etc/ssl/certs/"
    fi

fi

if grep -q "^PUBLIC_URL=http:" "$ENV_FILE";then
    sed -i 's/^PUBLIC_URL=http:/PUBLIC_URL=https:/' "$ENV_FILE"
fi
if grep -q "^SSO_PUBLIC_URL=http:" "$ENV_FILE";then
    sed -i 's/^SSO_PUBLIC_URL=http:/SSO_PUBLIC_URL=https:/' "$ENV_FILE"
fi
if grep -q "^\#\?JUPYTERHUB_CUSTOM_CA_CERTS" "$ENV_FILE";then
    sed -i "s,^\#\?JUPYTERHUB_CUSTOM_CA_CERTS.*$,JUPYTERHUB_CUSTOM_CA_CERTS=$PWD/ssl/deb_cacerts/," "$ENV_FILE"
fi

mkdir -p ssl/java_cacerts ssl/rh_cacerts ssl/deb_cacerts

echo "Fixing permissions on files"
sudo chmod -v a+w .env
if [ -d 'ssl/' ];then
    sudo chmod -v a+w ssl/
fi
if [ -d 'ssl/certificate.crt' ];then
    sudo chmod -v a+r ssl/certificate.crt
fi
if [ -d 'ssl/private.key' ];then
    sudo chmod -v a+r ssl/private.key
fi
